/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#define _WITH_DPRINTF
#include <stdio.h>
#include <unistd.h>
#include <glob.h>
#include <fstream>
#include "tuner_config.h"
#include "cx88_driver.h"
#include "cx88_debug.h"

using namespace std;

static ostream& generic_debug_timestamp(const char *component, ostream &stream)
{
   struct tm ltime;
   time_t curtime = time(NULL);
   localtime_r(&curtime, &ltime);
   char timebuf[32];
   strftime(timebuf, sizeof(timebuf), "%F %T", &ltime);
   return stream << "[" << component << " " << timebuf << "] ";
}

ostream& cx88_debug_timestamp(ostream &stream)
{
   return generic_debug_timestamp("cx88", stream);
}

static ostream& cx88_libtuner_logfunc(ostream *stream)
{
   *stream << CX88_DEBUG_NORM;
   return generic_debug_timestamp("libtuner", *stream);
}

static ostream& cx88_libtuner_errfunc(ostream *stream)
{
   *stream << CX88_DEBUG_ERR;
   return generic_debug_timestamp("libtuner", *stream);
}

struct cx88_libtuner_init
{
   cx88_libtuner_init()
   {
      libtuner_config::configure_log(&clog, cx88_libtuner_logfunc);
      libtuner_config::configure_err(&cerr, cx88_libtuner_errfunc);
   }
};

static cx88_libtuner_init libtuner_init;

const char *cx88_debug_color(const char *color)
{
   if (isatty(STDOUT_FILENO) && isatty(STDERR_FILENO))
   {
      return color;
   }
   else return "";
}

cx88_colorizer::cx88_colorizer(int fd, const char *color)
   : m_fd(fd)
{
   if (isatty(fd))
   {
      dprintf(fd, "%s", color);
   }
}

cx88_colorizer::~cx88_colorizer()
{
   if (isatty(m_fd))
   {
      dprintf(m_fd, "%s", CX88_DEBUG_COLOR_DEFAULT);
   }
}

void cx88_list_devices(cx88_xml_config &xmlconf, const char *header, const char *pattern)
{
   glob_t g;
   glob(pattern, 0, NULL, &g);
   char **paths = g.gl_pathv;
   if (header != NULL) cout << header << endl;
   ofstream temperr("/dev/null");
   streambuf *origerr = cerr.rdbuf(temperr.rdbuf());
   if (paths != NULL)
   {
      while (*paths != NULL)
      {
         cx88_xml_config::card_desc desc;
         if (cx88_driver::get_card_desc(xmlconf, *paths, desc) == 0)
         {
            cout << '\t' << *paths << ": ";
            if (desc.name == NULL) cout << "Unknown card (id: 0x" << hex << desc.id << dec << ")" << endl; 
            else cout << desc.name << endl;
         }
         ++paths;
      }
   }
   else cout << "\t(No devices found)" << endl;
   cerr.rdbuf(origerr);
   globfree(&g);
}

